<?php

namespace Drupal\monarch_media_bulk_upload\Form;

use Drupal\media\Entity\Media;
use Drupal\media\MediaInterface;
use Drupal\media_bulk_upload\Form\MediaBulkUploadForm;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Overrides the normal Media Bulk Upload form.
 */
class MediaBulkUploadFormOverride extends MediaBulkUploadForm {

  /**
   * Batch operation callback.
   *
   * @param string $id
   *   Batch operation id.
   * @param array $operation_details
   *   Batch operation details.
   * @param array $context
   *   Batch context.
   */
  public function batchOperation($id, array $operation_details, array &$context) {
    $mediaBulkConfig = $operation_details['media_bulk_config'];
    $mediaFormDisplay = $operation_details['media_form_display'];
    $file = $operation_details['file'];
    $form = $operation_details['form'];
    $form_state = $operation_details['form_state'];
    try {
      $media = $this->processFile($mediaBulkConfig, $file);
      if ($this->mediaSubFormManager->validateMediaFormDisplayUse($operation_details['media_bulk_config'])) {
        $extracted = $mediaFormDisplay->extractFormValues($media, $form['fields']['shared'], $form_state);
        $this->copyFormValuesToEntity($media, $extracted, $form_state);
      }
      $media->save();
      $context['results'][] = $media->id();

      $context['message'] = $this->t('Proccesing media @id.',
        [
          '@id' => $media->id(),
        ]
      );
    }
    catch (\Exception $e) {
      \watchdog_exception('media_bulk_upload', $e);
    }
  }

  /**
   * Batch finished callback.
   *
   * @param bool $success
   *   Batch success.
   * @param array $results
   *   Batch results.
   * @param array $operations
   *   Batch operations.
   */
  public function batchFinished($success, array $results, array $operations) {
    if ($success) {
      $this->messenger()->addMessage($this->t('@count media have been created.', ['@count' => count($results)]));
    }
    else {
      $errorOperation = reset($operations);
      $this->messenger()->addError(
        $this->t('An error occurred while processing @operation with arguments : @args',
          [
            '@operation' => $errorOperation[0],
            '@args' => print_r($errorOperation[0], TRUE),
          ]
        )
      );
    }

    $medias = array_filter(Media::loadMultiple($results));

    $media_ids = implode('+', array_map(function (MediaInterface $media) {
      return $media->id();
    }, $medias));

    return new RedirectResponse("/media-bulk-update/$media_ids");
  }

}
